"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mockFunctions = mockFunctions;
exports.default = void 0;

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function constructBasedOff(obj) {
  if (obj.constructor) {
    return new obj.constructor();
  } else {
    // We can fallback to just cloning the prototype across if there's no constructor
    return objectWithPrototypeFrom(obj);
  }
}

function objectWithPrototypeFrom(obj) {
  return Object.assign(Object.create(Object.getPrototypeOf(obj)), obj);
}

function mockValue(realValue, options, ogToMockedMap) {
  var recursive = options.recursive;
  var classInstances = recursive ? recursive.classInstances : false;

  if (ogToMockedMap.has(realValue)) {
    return ogToMockedMap.get(realValue);
  }

  function handleContainer(createMockStubCallback, mockCallback) {
    // .size == 0 to see if we haven't done any recursion yet
    if (options.recursive || ogToMockedMap.size == 0) {
      var mocked = createMockStubCallback(realValue);
      ogToMockedMap.set(realValue, mocked);
      return mockCallback(realValue, mocked, options, ogToMockedMap);
    } else {
      return realValue;
    }
  }

  if (typeof realValue === 'function') {
    if (ogToMockedMap.has(realValue)) {}

    var mockedFn = jest.fn();
    ogToMockedMap.set(realValue, mockedFn);

    if (options.onMockedFunction) {
      options.onMockedFunction(mockedFn, realValue);
    }

    return mockedFn;
  } else if (realValue === null || realValue === undefined) {
    return realValue;
  } else if (realValue instanceof Map) {
    return handleContainer(constructBasedOff, mockMapValues);
  } else if (realValue instanceof Set) {
    return handleContainer(constructBasedOff, mockSetValues);
  } else if (Array.isArray(realValue) || Object.getPrototypeOf({}) === Object.getPrototypeOf(realValue) || Object.getPrototypeOf(exports) === Object.getPrototypeOf(realValue)) {
    // Mock fields in {...}
    return handleContainer(constructBasedOff, mockProperties);
  } else if (_typeof(realValue) === 'object' && (classInstances || ogToMockedMap.size == 0)) {
    return handleContainer(objectWithPrototypeFrom, function (realVal, mocked, opts, map) {
      var mocked2 = mockPrototypeFunctions(realVal, mocked, opts, map);
      return mockProperties(realVal, mocked2, opts, map);
    });
  } else {
    return realValue;
  }
}

function mockPrototypeFunctions(value, mockedObject) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var ogToMockedMap = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : new Map();
  // This will map the prototype values to the mocked object and not other objects with the same prototype
  var prototype = Object.getPrototypeOf(value);

  if (prototype) {
    return mockProperties(prototype, mockedObject, options, ogToMockedMap);
  } else {
    return mockedObject;
  }
}

function mockProperties(value, mockedObject) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var ogToMockedMap = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : new Map();
  var propertyDescriptors = Object.getOwnPropertyDescriptors(value);
  Object.keys(propertyDescriptors).forEach(function (key) {
    var propertyDescriptor = propertyDescriptors[key];
    var mockedPropertyDescriptor = propertyDescriptor.get || propertyDescriptor.set ? _objectSpread({}, propertyDescriptor, {
      get: mockValue(propertyDescriptor.get, options, ogToMockedMap),
      set: mockValue(propertyDescriptor.set, options, ogToMockedMap)
    }) : _objectSpread({}, propertyDescriptor, {
      value: mockValue(propertyDescriptor.value, options, ogToMockedMap)
    });
    Object.defineProperty(mockedObject, key, mockedPropertyDescriptor);
  });
  return mockedObject;
}

function mockMapValues(actualMap, mockedMap) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var ogToMockedMap = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : new Map();
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = actualMap.entries()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _step$value = _slicedToArray(_step.value, 2),
          key = _step$value[0],
          value = _step$value[1];

      mockedMap.set(key, mockValue(value, options, ogToMockedMap));
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return mockedMap;
}

function mockSetValues(actualSet, mockedSet) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var ogToMockedMap = arguments.length > 3 ? arguments[3] : undefined;
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = actualSet[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var value = _step2.value;
      mockedSet.add(mockValue(value, options, ogToMockedMap));
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return mockedSet;
}

function mockFunctions(value) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var ogToMockedMap = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : new Map();
  return mockValue(value, options, ogToMockedMap);
}

var _default = mockFunctions;
exports.default = _default;
//# sourceMappingURL=index.js.map